#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Suhail"
__credits__ = ["Suhail"]
__Lisence__ = "MIT"
__maintainer__ = "Suhail"
__email__ = "suhahilece@gmail.com"
__status__ = "Development"
__version__ = "0.1.0"

"""
tex_id_manager.py
-----------------
This module contains the TextureIDManager class for managing texture ID patterns.

Author: Suhail
License: MIT
"""

# Default python packages


import os
import json
import logging

# Configure logging
logging.basicConfig(level=logging.INFO)

class TextureIDManager:
    def __init__(self, config_file: str = 'textureID.json', config_file_path: str = None):
        if config_file_path:
            self.config_file = os.path.join(config_file_path, config_file)
        else:
            self.config_file = os.path.join(os.getcwd(), config_file)
        
        self.textureType = self._load_texture_types()
        self.asset_name = ''
        self._get_texture_types()
        
    def _load_texture_types(self):
        try:
            with open(self.config_file, 'r') as f:
                return json.load(f)
        except FileNotFoundError:
            logging.error(f"Configuration file {self.config_file} not found.")
            return {}
        except json.JSONDecodeError:
            logging.error(f"Error decoding JSON from configuration file {self.config_file}.")
            return {}

    def _get_texture_types(self):
        import hou
        message = 'Set Texture ID names. No Spaces Allowed!'
        input_labels = ["Asset Name:"] + [f"{data['label']}:" for data in self.textureType.values()]
        initial_contents = [''] + [f"{data['pattern']}" for data in self.textureType.values()]

        input_labels = tuple(input_labels)
        initial_contents = tuple(initial_contents)

        title = 'Set Texture ID names'
        buttons = ('OK', 'Cancel')
        default_choice = 0
        close_choice = 1
        
        user_input_button, user_input_values = hou.ui.readMultiInput(
            message=message,
            input_labels=input_labels,
            initial_contents=initial_contents,
            title=title,
            buttons=buttons,
            default_choice=default_choice,
            close_choice=close_choice
        )

        self.user_input_button = user_input_button
        self.user_input_values = user_input_values

        if user_input_button == default_choice:
            self.asset_name = user_input_values[0]  # Asset Name is the first input
            updated = False
            for idx, key in enumerate(self.textureType.keys()):
                new_value = user_input_values[idx + 1]  # +1 to skip 'Asset Name'
                if self.textureType[key]['pattern'] != new_value:
                    self.textureType[key]['pattern'] = new_value
                    updated = True
            if updated:
                self._save_texture_types()

    def _save_texture_types(self):
        try:
            with open(self.config_file, 'w') as f:
                json.dump(self.textureType, f, indent=4)
        except Exception as e:
            logging.error(f"Error saving configuration to {self.config_file}: {e}")



import os
import site
import json
import platform
import re

def get_houdini_versions(base_dir, pattern):
    versions = []
    if os.path.exists(base_dir):
        for item in os.listdir(base_dir):
            match = re.match(pattern, item)
            if match:
                versions.append(match.group(1))
    return versions

def get_latest_houdini_version(base_dir, pattern):
    versions = get_houdini_versions(base_dir, pattern)
    if versions:
        return max(versions, key=lambda v: list(map(int, v.split('.'))))
    return None

def get_current_houdini_version(base_dir, pattern):
    current_path = os.path.join(base_dir, 'Current')
    if os.path.islink(current_path):
        actual_path = os.readlink(current_path)
        version = os.path.basename(actual_path)
        match = re.match(pattern, version)
        if match:
            return match.group(1)
    return None

def get_houdini_major_version():
    system = platform.system()
    if system == "Windows":
        base_dir = "C:\\Program Files\\Side Effects Software"
        pattern = r'Houdini (\d+\.\d+)\.\d+'
        check_current = False
    elif system == "Darwin":
        base_dir = "/Applications/Houdini"
        pattern = r'Houdini(\d+\.\d+)\.\d+'
        check_current = True
    else:
        print("Unsupported operating system.")
        return None

    latest_version = get_latest_houdini_version(base_dir, pattern)

    if latest_version:
        return latest_version
    elif check_current:
        current_version = get_current_houdini_version(base_dir, pattern)
        if current_version:
            return current_version

    return None

def post_install():
    print("Running post install setup...")
    houdini_version = get_houdini_major_version()
    if not houdini_version:
        print("No Houdini versions found.")
        return

    user_pref_dir = ''
    if platform.system() == 'Windows':
        onedrive_dir = os.path.expanduser(f'~/OneDrive/Documents/houdini{houdini_version}')
        default_dir = os.path.expanduser(f'~/Documents/houdini{houdini_version}')
        user_pref_dir = onedrive_dir if os.path.exists(onedrive_dir) else default_dir
    elif platform.system() == 'Darwin':
        user_pref_dir = os.path.expanduser(f'~/Library/Preferences/Houdini/{houdini_version}')

    print(f"Determined Houdini user preference directory: {user_pref_dir}")

    toolbar_dir = os.path.join(user_pref_dir, 'toolbar')
    packages_dir = os.path.join(user_pref_dir, 'packages')
    shelf_file_path = os.path.join(toolbar_dir, 'houdiniUtils.shelf')
    json_file_path = os.path.join(packages_dir, 'houdiniUtils.json')

    print(f"Toolbar directory: {toolbar_dir}")
    print(f"Packages directory: {packages_dir}")
    print(f"Shelf file path: {shelf_file_path}")
    print(f"JSON file path: {json_file_path}")

    os.makedirs(toolbar_dir, exist_ok=True)
    os.makedirs(packages_dir, exist_ok=True)

    # Updated shelf content
    shelf_content = '''<?xml version="1.0" encoding="UTF-8"?>
<shelfDocument>
  <toolshelf name="houdiniUtils" label="Houdini Utils">
    <memberTool name="ren_tex_files1"/>
  </toolshelf>
  <tool name="ren_tex_files1" label="Ren Texture" icon="hicon:/SVGIcons.index?SOP_grouprename.svg">
    <script scriptType="python"><![CDATA[__author__ = "Suhail"
__credits__ = ["Suhail"]
__Lisence__ = "MIT"
__maintainer__ = "Suhail"
__email__ = "suhailece@gmail.com"
__status__ = "Development"
__version__ = "0.1.5"

from houdiniutils.textureTools import main
from importlib import reload

reload(main)
main.main()]]></script>
  </tool>
  <tool name="ren_tex_files" label="Ren Texture" icon="hicon:/SVGIcons.index?SOP_grouprename.svg">
    <script scriptType="python"><![CDATA[from houdiniutils.textureTools import main
from importlib import reload

reload(main)
main.main()]]></script>
  </tool>
</shelfDocument>'''

    with open(shelf_file_path, 'w') as f:
        f.write(shelf_content)

    print(f"Written shelf file to {shelf_file_path}")

    site_packages_path = [p.replace("\\", "/") for p in site.getsitepackages() if "site-packages" in p][0]
    config = {"env": [{"PYTHONPATH": [site_packages_path]}]}
    with open(json_file_path, 'w') as f:
        json.dump(config, f, indent=4)

    print(f"Written JSON file to {json_file_path}")

if __name__ == "__main__":
    post_install()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Suhail"
__credits__ = ["Suhail"]
__Lisence__ = "MIT"
__maintainer__ = "Suhail"
__email__ = "suhahilece@gmail.com"
__status__ = "Development"
__version__ = "0.1.4"

"""
renameTexture.py
----------------
This module contains the RenameTexture class for renaming texture files based on specified patterns.

Author: Suhail
License: MIT
"""

# Default python packages

import os
import re
import json
import logging
import platform

# Configure logging
logging.basicConfig(level=logging.INFO)

class RenameTexture:
    def __init__(self, sourceFolder: str = None, old_file_name: bool = True, asset_name: str = None, **kwargs):
        self.old_file_name = old_file_name
        self.asset_name = asset_name
        if not sourceFolder:
            self._get_source_folder()
        else:
            self.sourceFolder = sourceFolder
        self.textureTypes = kwargs.get('textureTypes', {})

    def _get_source_folder(self):
        import hou
        start_directory = hou.getenv('HIP')
        title = 'Select root folder'
        file_type = hou.fileType.Directory
        sourceFolder = hou.ui.selectFile(start_directory=start_directory, title=title, file_type=file_type)
        self.sourceFolder = hou.text.expandString(sourceFolder) if sourceFolder else None

    def renameFolders(self):
        if not self.sourceFolder:
            logging.info('No source folder selected. Operation cancelled.')
            return

        try:
            is_single_folder = all(
                os.path.isfile(os.path.join(self.sourceFolder, item)) and not self._is_hidden_or_system_file(os.path.join(self.sourceFolder, item))
                for item in os.listdir(self.sourceFolder)
            )
            if is_single_folder:
                folder_name = os.path.basename(self.sourceFolder)
                logging.info(f"Renaming files in single folder: {self.sourceFolder}, folder name: {folder_name}")
                self._rename_files_in_folder(self.sourceFolder, folder_name)
            else:
                for root, dirs, files in os.walk(self.sourceFolder):
                    # Filter out hidden and system files and directories
                    dirs[:] = [d for d in dirs if not self._is_hidden_or_system_file(os.path.join(root, d))]
                    files[:] = [f for f in files if not self._is_hidden_or_system_file(os.path.join(root, f))]
                    
                    if root == self.sourceFolder:
                        for subdir in dirs:
                            subdir_path = os.path.join(root, subdir)
                            self._rename_folder_and_files(subdir_path)
        except Exception as e:
            logging.error(f"Error during renaming folders: {e}")

    def _is_hidden_or_system_file(self, file_path):
        if platform.system() == 'Windows':
            import ctypes
            FILE_ATTRIBUTE_HIDDEN = 0x02
            FILE_ATTRIBUTE_SYSTEM = 0x04
            attrs = ctypes.windll.kernel32.GetFileAttributesW(file_path)
            if attrs == -1:
                return False
            return bool(attrs & (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM))
        else:
            return os.path.basename(file_path).startswith('.')

    def _rename_folder_and_files(self, folder_path):
        parent_folder = os.path.dirname(folder_path)
        folder_name = os.path.basename(folder_path)
        new_folder_name = re.sub(r'[.\s\-\(\)\+]', '_', folder_name)
        new_folder_name = re.sub(r'__+', '_', new_folder_name)
        logging.info(f"Processing folder: {folder_path}, new folder name: {new_folder_name}")
        if not new_folder_name:
            logging.error(f"New folder name is empty for folder: {folder_path}")
            return

        new_folder_path = os.path.join(parent_folder, new_folder_name)
        
        try:
            if folder_path != new_folder_path:
                os.rename(folder_path, new_folder_path)
                logging.info(f"Renamed folder {folder_path} to {new_folder_path}")
                folder_path = new_folder_path
        except PermissionError:
            logging.error(f"Permission denied while renaming folder {folder_path} to {new_folder_path}")
            return
        except Exception as e:
            logging.error(f"Error renaming folder {folder_path} to {new_folder_path}: {e}")
            return
        
        self._rename_files_in_folder(folder_path, new_folder_name)

    def _rename_files_in_folder(self, folder_path, folder_name):
        rename_plan = {}  # Dictionary to plan renaming
        rename_log = []   # List to store old and new file names for JSON log

        try:
            # First pass: collect rename plan and count occurrences
            for file in os.listdir(folder_path):
                if self._is_hidden_or_system_file(os.path.join(folder_path, file)):
                    continue  # Skip hidden and system files
                file_path = os.path.join(folder_path, file)
                if os.path.isfile(file_path):
                    file_name, file_ext = os.path.splitext(file)
                    for key, data in self.textureTypes.items():
                        patterns = [pattern.strip() for pattern in data['pattern'].split(',')]
                        regex_patterns = [pattern.replace('*', '.*') for pattern in patterns]
                        regex_pattern = '|'.join(regex_patterns)
                        if re.search(regex_pattern, file_name, re.IGNORECASE):
                            new_file_base = f'{self.asset_name}_{folder_name}_{key}' if self.asset_name else f'{folder_name}_{key}'
                            if new_file_base not in rename_plan:
                                rename_plan[new_file_base] = []
                            rename_plan[new_file_base].append((file_name, file_ext))
                            break

            # Second pass: rename files according to the plan
            for new_file_base, files in rename_plan.items():
                count = len(files)
                for index, (file_name, file_ext) in enumerate(files):
                    old_file_path = os.path.join(folder_path, file_name + file_ext)
                    if count > 1:
                        new_file_name = f'{new_file_base}{index + 1}{file_ext}'
                    else:
                        new_file_name = f'{new_file_base}{file_ext}'
                    new_file_path = os.path.join(folder_path, new_file_name)
                    
                    if self.old_file_name:
                        rename_log.append({"old_name": file_name + file_ext, "new_name": new_file_name})

                    try:
                        os.rename(old_file_path, new_file_path)
                        logging.info(f"Renamed {file_name + file_ext} to {new_file_name}")
                    except PermissionError:
                        logging.error(f"Permission denied while renaming file {old_file_path} to {new_file_path}")
                    except Exception as e:
                        logging.error(f"Error renaming file {old_file_path} to {new_file_path}: {e}")

            if self.old_file_name and rename_log:
                log_file_path = os.path.join(folder_path, f'{folder_name}_rename_log.json')
                try:
                    with open(log_file_path, 'w') as log_file:
                        json.dump(rename_log, log_file, indent=4)
                    logging.info(f"Rename log saved to {log_file_path}")
                except PermissionError:
                    logging.error(f"Permission denied while saving rename log to {log_file_path}")
                except Exception as e:
                    logging.error(f"Error saving rename log to {log_file_path}: {e}")

        except PermissionError:
            logging.error(f"Permission denied while accessing folder {folder_path}")
        except Exception as e:
            logging.error(f"Error during renaming files in folder {folder_path}: {e}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Suhail"
__credits__ = ["Suhail"]
__Lisence__ = "MIT"
__maintainer__ = "Suhail"
__email__ = "suhahilece@gmail.com"
__status__ = "Development"
__version__ = "0.1.0"

"""
main.py
-------
Main entry point for the texture renaming utility.

Author: Suhail
License: MIT
"""

# Default python packages

import os
from houdiniutils.textureTools import tex_id_manager, renameTexture

def main():
    # Set the config file path
    config_file_path = os.path.dirname(__file__)

    # Initialize TextureIDManager
    texid = tex_id_manager.TextureIDManager(config_file_path=config_file_path)

    # Initialize RenameTexture with textureTypes and asset name
    renObj = renameTexture.RenameTexture(textureTypes=texid.textureType, asset_name=texid.asset_name)
    renObj.renameFolders()

if __name__ == "__main__":
    main()
